'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var createCache = _interopDefault(require('@emotion/cache'));
var utils = require('@emotion/utils');
var serialize = require('@emotion/serialize');
var sheet = require('@emotion/sheet');
var css = _interopDefault(require('@emotion/css'));

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var BasicProvider =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(BasicProvider, _React$Component);

  function BasicProvider(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      value: createCache()
    };
    return _this;
  }

  var _proto = BasicProvider.prototype;

  _proto.render = function render() {
    return React.createElement(CSSContext.Provider, this.state, this.props.children(this.state.value));
  };

  return BasicProvider;
}(React.Component);

var CSSContext = React.createContext(process.env.NODE_ENV !== 'test' ? createCache() : null);
function withCSSContext(func) {
  return function (props) {
    return React.createElement(CSSContext.Consumer, null, function (context) {
      if (context === null) {
        return React.createElement(BasicProvider, null, function (newContext) {
          return func(props, newContext);
        });
      } else {
        return func(props, context);
      }
    });
  };
}
function consume(func) {
  return React.createElement(CSSContext.Consumer, null, function (context) {
    if (context === null) {
      return React.createElement(BasicProvider, null, function (newContext) {
        return func(newContext);
      });
    } else {
      return func(context);
    }
  });
}

var jsx = function jsx(type, props) {
  var _arguments = arguments;

  if (props == null || props.css == null) {
    // $FlowFixMe
    return React.createElement.apply(undefined, arguments);
  }

  if (typeof props.css === 'string' && // check if there is a css declaration
  props.css.indexOf(':') !== -1 && process.env.NODE_ENV !== 'production') {
    throw new Error("Strings are not allowed as css prop values, please wrap it in a css template literal from '@emotion/css' like this: css`" + props.css + "`");
  }

  return consume(function (context) {
    var registeredStyles = [];
    var className = '';

    if (props.className !== undefined) {
      className = utils.getRegisteredStyles(context.registered, registeredStyles, props.className);
    }

    registeredStyles.push(typeof props.css === 'function' ? props.css(context.theme) : props.css);
    var serialized = serialize.serializeStyles(context.registered, registeredStyles);
    var rules = utils.insertStyles(context, serialized);
    className += context.key + "-" + serialized.name;
    var newProps = {};

    for (var key in props) {
      if (Object.prototype.hasOwnProperty.call(props, key) && key !== 'css') {
        newProps[key] = props[key];
      }
    }

    newProps.className = className;
    var argsLength = _arguments.length;
    var createElementArgArray = new Array(argsLength);
    createElementArgArray[0] = type;
    createElementArgArray[1] = newProps;

    for (var i = 2; i < argsLength; i++) {
      createElementArgArray[i] = _arguments[i];
    } // $FlowFixMe


    var ele = React.createElement.apply(undefined, createElementArgArray);

    return ele;
  });
};

var Global =
/* #__PURE__ */
withCSSContext(function (props, context) {
  return React.createElement(InnerGlobal, {
    styles: props.styles,
    context: context
  });
});

// maintain place over rerenders.
// initial render from browser, insertBefore context.sheet.tags[0] or if a style hasn't been inserted there yet, appendChild
// initial client-side render from SSR, use place of hydrating tag
var InnerGlobal =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(InnerGlobal, _React$Component);

  function InnerGlobal(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = InnerGlobal.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.updateStyles();
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.updateStyles();
  };

  _proto.updateStyles = function updateStyles() {
    var _this = this;

    {
      var serialized = serialize.serializeStyles(this.props.context.registered, [this.props.styles]);

      if (serialized.name === this.styleName) {
        return;
      }

      this.styleName = serialized.name;

      if (!this.sheet) {
        this.sheet = new sheet.StyleSheet({
          key: this.props.context.key + "-global",
          // $FlowFixMe
          nonce: this.props.context.sheet.nonce,
          container: this.props.context.sheet.container
        }); // $FlowFixMe

        var node = document.querySelector("style[data-emotion-" + this.props.context.key + "=\"" + serialized.name + "\"]");

        if (node !== null) {
          this.sheet.tags.push(node);
        } // $FlowFixMe


        if (this.props.context.sheet.tags.length) {
          this.sheet.before = this.props.context.sheet.tags[0];
        }
      }

      var rules = this.props.context.stylis("", serialized.styles);

      if (this.sheet.tags.length) {
        // if this doesn't exist then it will be null so the style element will be appended
        this.sheet.before = this.sheet.tags[0].nextElementSibling;
        this.sheet.flush();
      }

      rules.forEach(function (rule) {
        _this.sheet.insert(rule);
      });
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.sheet.flush();
  };

  _proto.render = function render() {

    return null;
  };

  return InnerGlobal;
}(React.Component);

var keyframes = function keyframes(arg) {
  var name = "animation-" + arg.name;
  return {
    name: name,
    styles: "@keyframes " + name + "{" + arg.styles + "}"
  };
};

exports.css = css;
exports.jsx = jsx;
exports.Global = Global;
exports.keyframes = keyframes;
exports.CSSContext = CSSContext;
exports.withCSSContext = withCSSContext;
exports.consume = consume;
//# sourceMappingURL=index.browser.cjs.js.map
