import { isBrowser, getRegisteredStyles, insertStyles, shouldSerializeToReactTree } from '@emotion/utils';
import { createElement, Component, createContext, Fragment } from 'react';
import createCache from '@emotion/cache';
import { serializeStyles } from '@emotion/serialize';
import { StyleSheet } from '@emotion/sheet';
export { default as css } from '@emotion/css';

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var BasicProvider =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(BasicProvider, _React$Component);

  function BasicProvider(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.state = {
      value: createCache()
    };
    return _this;
  }

  var _proto = BasicProvider.prototype;

  _proto.render = function render() {
    return createElement(CSSContext.Provider, this.state, this.props.children(this.state.value));
  };

  return BasicProvider;
}(Component);

var CSSContext = createContext(isBrowser && process.env.NODE_ENV !== 'test' ? createCache() : null);
function withCSSContext(func) {
  return function (props) {
    return createElement(CSSContext.Consumer, null, function (context) {
      if (context === null) {
        return createElement(BasicProvider, null, function (newContext) {
          return func(props, newContext);
        });
      } else {
        return func(props, context);
      }
    });
  };
}
function consume(func) {
  return createElement(CSSContext.Consumer, null, function (context) {
    if (context === null) {
      return createElement(BasicProvider, null, function (newContext) {
        return func(newContext);
      });
    } else {
      return func(context);
    }
  });
}

var jsx = function jsx(type, props) {
  var _arguments = arguments;

  if (props == null || props.css == null) {
    // $FlowFixMe
    return createElement.apply(undefined, arguments);
  }

  if (typeof props.css === 'string' && // check if there is a css declaration
  props.css.indexOf(':') !== -1 && process.env.NODE_ENV !== 'production') {
    throw new Error("Strings are not allowed as css prop values, please wrap it in a css template literal from '@emotion/css' like this: css`" + props.css + "`");
  }

  return consume(function (context) {
    var registeredStyles = [];
    var className = '';

    if (props.className !== undefined) {
      className = getRegisteredStyles(context.registered, registeredStyles, props.className);
    }

    registeredStyles.push(typeof props.css === 'function' ? props.css(context.theme) : props.css);
    var serialized = serializeStyles(context.registered, registeredStyles);
    var rules = insertStyles(context, serialized);
    className += context.key + "-" + serialized.name;
    var newProps = {};

    for (var key in props) {
      if (Object.prototype.hasOwnProperty.call(props, key) && key !== 'css') {
        newProps[key] = props[key];
      }
    }

    newProps.className = className;
    var argsLength = _arguments.length;
    var createElementArgArray = new Array(argsLength);
    createElementArgArray[0] = type;
    createElementArgArray[1] = newProps;

    for (var i = 2; i < argsLength; i++) {
      createElementArgArray[i] = _arguments[i];
    } // $FlowFixMe


    var ele = createElement.apply(undefined, createElementArgArray);

    if (shouldSerializeToReactTree && rules !== undefined) {
      var _ref;

      return createElement(Fragment, null, createElement("style", (_ref = {}, _ref["data-emotion-" + context.key] = serialized.name, _ref.dangerouslySetInnerHTML = {
        __html: rules
      }, _ref)), ele);
    }

    return ele;
  });
};

var Global =
/* #__PURE__ */
withCSSContext(function (props, context) {
  return createElement(InnerGlobal, {
    styles: props.styles,
    context: context
  });
});

// maintain place over rerenders.
// initial render from browser, insertBefore context.sheet.tags[0] or if a style hasn't been inserted there yet, appendChild
// initial client-side render from SSR, use place of hydrating tag
var InnerGlobal =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(InnerGlobal, _React$Component);

  function InnerGlobal(props) {
    return _React$Component.call(this, props) || this;
  }

  var _proto = InnerGlobal.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.updateStyles();
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.updateStyles();
  };

  _proto.updateStyles = function updateStyles() {
    var _this = this;

    if (shouldSerializeToReactTree === false) {
      var serialized = serializeStyles(this.props.context.registered, [this.props.styles]);

      if (serialized.name === this.styleName) {
        return;
      }

      this.styleName = serialized.name;

      if (!this.sheet) {
        this.sheet = new StyleSheet({
          key: this.props.context.key + "-global",
          // $FlowFixMe
          nonce: this.props.context.sheet.nonce,
          container: this.props.context.sheet.container
        }); // $FlowFixMe

        var node = document.querySelector("style[data-emotion-" + this.props.context.key + "=\"" + serialized.name + "\"]");

        if (node !== null) {
          this.sheet.tags.push(node);
        } // $FlowFixMe


        if (this.props.context.sheet.tags.length) {
          this.sheet.before = this.props.context.sheet.tags[0];
        }
      }

      var rules = this.props.context.stylis("", serialized.styles);

      if (this.sheet.tags.length) {
        // if this doesn't exist then it will be null so the style element will be appended
        this.sheet.before = this.sheet.tags[0].nextElementSibling;
        this.sheet.flush();
      }

      rules.forEach(function (rule) {
        _this.sheet.insert(rule);
      });
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.sheet.flush();
  };

  _proto.render = function render() {
    if (shouldSerializeToReactTree) {
      var _ref;

      var serialized = serializeStyles(this.props.context.registered, [this.props.styles]);
      var rules = this.props.context.stylis("", serialized.styles);
      return createElement("style", (_ref = {}, _ref["data-emotion-" + this.props.context.key] = serialized.name, _ref.dangerouslySetInnerHTML = {
        __html: rules.join('')
      }, _ref));
    }

    return null;
  };

  return InnerGlobal;
}(Component);

var keyframes = function keyframes(arg) {
  var name = "animation-" + arg.name;
  return {
    name: name,
    styles: "@keyframes " + name + "{" + arg.styles + "}"
  };
};

export { jsx, Global, keyframes, CSSContext, withCSSContext, consume };
//# sourceMappingURL=index.esm.js.map
